
// SPGGenomePool.cpp
// Copyright 2006 Steve Gargolinski

/*
This file is part of SPGGenomePool.

SPGGenomePool is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

SPGGenomePool is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SPGGenomePool; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include "SPGGenomePool.h"


// Here are a bunch of defines that you can use to tweak the behavior of this genome pool
// SPGTODO:  Add a second constructor to set these things

#define SPGGENOMEPOOL_POOLSIZE			150
#define SPGGENOMEPOOL_MUTATIONRATE		0.001f
#define SPGGENOMEPOOL_CROSSOVERRATE		0.7f
#define SPGGENOMEPOOL_NUMCHROMOSOMEBITS 100

//
// Constructor
//
SPGGenomePool::SPGGenomePool()
{
	// Just init this with the default values
	m_poolSize = SPGGENOMEPOOL_POOLSIZE;
	m_mutationRate = SPGGENOMEPOOL_MUTATIONRATE;
	m_crossoverRate = SPGGENOMEPOOL_CROSSOVERRATE;
	m_numChromosomeBits = SPGGENOMEPOOL_NUMCHROMOSOMEBITS;

	// Always start on the 0th generation
	m_generation = 0;

	// Clear out the genome list
	m_genomeList.clear();

	// No current strongest Genome
	m_currentStrongestGenome = NULL;
}

//
// Destructor needs to make sure to clean up all of the SPGGenomes in this pool
//
SPGGenomePool::~SPGGenomePool()
{
	// Make sure we clear out the pool
	ClearPool();
}

//
// Goes through and delete all of the genomes in this pool
//
void SPGGenomePool::ClearPool()
{
	SPGGenome* deleteMe = NULL;

	// Loop through all of the genomes in here
	for (size_t i = 0; i < m_genomeList.size(); ++i) 
	{	
		// And delete each one
		deleteMe = m_genomeList[i];

		if (deleteMe != NULL) 
		{
			delete deleteMe;
			deleteMe = NULL;
		}
	}

	m_genomeList.clear();
}

//
// Creates an initial population in this pool
//
void SPGGenomePool::CreateInitialPool()
{
	// First clear out the pool
	ClearPool();

	for (size_t i = 0; i < m_poolSize; ++i)
	{
		// Create a new genome
		SPGGenome* newGenome = new SPGGenome(m_numChromosomeBits);

		// Fill it up with random bits for now
		newGenome->FillGenomeBinary();

		// And now add it to the list
		m_genomeList.push_back(newGenome);
	}
}

//
// 'Breeds' the passed in mother and father, resulting in two new SPGGenomes
// Returns true if we actually did a crossover, false otherwise
//
bool SPGGenomePool::DoCrossover(SPGGenome* mother, SPGGenome* father, SPGGenome*& newChild0, SPGGenome*& newChild1)
{
	// First make sure that the mother and father are valid
	// And that they don't point to the same SPGGenome
	if (mother == NULL || father == NULL || mother == father)
	{
		// Make sure we NULL out the children to be safe and then return false
		newChild0 = NULL;
		newChild1 = NULL;

		return false;
	}

	// Now check to see if we're going to do any sort of crossover
	float r = rand() / (RAND_MAX + 1.0f);
	
	// If we make it inside this if then we're NOT going to do a crossover
	if (r > m_crossoverRate)
	{
		// So we'll set the new children to be just like the parents
		const std::vector<int> chromo0 = mother->GetData();
		const std::vector<int> chromo1 = father->GetData();

		// Create the two new children
		SPGGenome* g0 = new SPGGenome(chromo0);
		SPGGenome* g1 = new SPGGenome(chromo1);

		newChild0 = g0;
		newChild1 = g1;

		// We didn't do a crossover, so return false
		return false;
	}


	// Ok this means we want to do the crossover
	// first create the new children
	SPGGenome* g0 = new SPGGenome();
	SPGGenome* g1 = new SPGGenome();

	// We need to start out with the mother and father chromosomes
	const std::vector<int> motherData = mother->GetData();
	const std::vector<int> fatherData = father->GetData();
	
	// And let's start out with empty chromosomes for the children
	std::vector<int> child0Data;
	std::vector<int> child1Data;
	child0Data.clear();
	child1Data.clear();

	// Now choose a random location in the chromosome to split
	int split = rand() % m_numChromosomeBits;

	// Now let's go through and copy over the chromosomes, doing the crossover
	// in the correct spot
	for (size_t i = 0; i < (size_t)split; ++i)
	{
		child0Data.push_back(motherData[i]);
		child1Data.push_back(fatherData[i]);
	}
	for (size_t i = split; i < m_numChromosomeBits; ++i)
	{
		child0Data.push_back(fatherData[i]);
		child1Data.push_back(motherData[i]);
	}

	// Now assign this data to our new children
	g0->SetData(child0Data);
	g1->SetData(child1Data);

	// Make sure the output parameters are done ok
	newChild0 = g0;
	newChild1 = g1;

	// If we make it down here then we did a crossover, return true
	return true;
}

//
// Chooses a semi-random SPGGenome from this pool, weighted so that the stronger
// ones will have a higher chance of being picked in a simple roulette wheel selection
// 
// If different selection strategies are used further down the line then swap this
// function out with a strategy pattern
//
SPGGenome* SPGGenomePool::ChooseGenome()
{
	// First total up the strength of this entire pool
	float totalPoolStrength = 0.0f;
	for (size_t i = 0; i < m_genomeList.size(); ++i) 
	{
		SPGGenome* genome = m_genomeList[i];

		if (genome != NULL) 
		{
			totalPoolStrength += genome->GetStrength();
		}
	}

	// Now get a random position between 0 and this total
	float randPos = (rand() / (RAND_MAX + 1.0f)) * totalPoolStrength;
	float currentTotal = 0.0f;
	
	// Now loop through the genome list and see where this random position lies
	for (size_t i = 0; i < m_genomeList.size(); ++i)
	{
		SPGGenome* genome = m_genomeList[i];

		if (genome != NULL)
		{
			currentTotal += genome->GetStrength();
		}

		// Check to see if we have gone far enough and we can return this SPGGenome
		if (currentTotal > randPos)
		{
			return genome;
		}
	}

	// If we make it down here then something went wrong so return NULL
	return NULL;
}

//
// Runs a generation of this genome pool
//
void SPGGenomePool::RunGeneration()
{
	// First make sure that we do have an actual population of genomes here
	if (m_genomeList.size() <= 0)
	{
		return;
	}

	// The first thing we want to do is run the heuristic and see how
	// strong all of the genomes in this pool are
	CalcGenomeStrengths();

	// Next we need to repeat the following until we have a new full pool:
	//	- Choose two random members of the population and 'breed' them
	//	- Mutate the new chromosomes
	
	// Keep a list of the new children
	std::vector<SPGGenome*> newChildren;
	newChildren.clear();
	
	// Keep looping until we have enough children to make up a new pool
	while (newChildren.size() < m_poolSize)
	{
		// Choose a mother and a father
		SPGGenome* mother = ChooseGenome();
		SPGGenome* father = ChooseGenome();

		// Get pointers to the soon-to-be children
		SPGGenome* newChild0 = NULL;
		SPGGenome* newChild1 = NULL;

		// Do the crossover between the mother and the father
		if (DoCrossover(mother, father, newChild0, newChild1))
		{
			// Now we need to mutate the children
			newChild0->Mutate(m_mutationRate);
			newChild1->Mutate(m_mutationRate);

			// Now push the new children onto the new children list
			newChildren.push_back(newChild0);
			newChildren.push_back(newChild1);
		}
	}

	// Now that we've finished, delete out all of the old children
	// And set the list to this new array of pointers to SPGGenomes
	ClearPool();
	m_genomeList = newChildren;

	// We need to update the strengths with the new children
	CalcGenomeStrengths();

	// Finally we increment the generation counter
	m_generation += 1;
}