

// SPGGenomePool.h
// Copyright 2006 Steve Gargolinski

/*
This file is part of SPGGenomePool.

SPGGenomePool is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

SPGGenomePool is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with SPGGenomePool; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef __SPGGENOMEPOOL_H__
#define __SPGGENOMEPOOL_H__

#include <vector>
#include "SPGGenome.h"

//
// SPGGenomePool is a collection of SPGGenomes which undergoes mutations, combinations
// through generations, and general evolution
//
// The real thing you need to do here if you're going to use this class is to make sure
// that you override CalcGenomeStrengths().  This is the function that differentiates
// one genetic algorithm from another.
//

class SPGGenomePool
{
public:

	//
	// Constructor
	//
	SPGGenomePool();

	//
	// Destructor needs to make sure to clean up all of the SPGGenomes in this pool
	//
	~SPGGenomePool();



	//
	// Returns the size of this number of genomes in this pool
	//
	unsigned int GetPoolSize() { return m_poolSize;	}

	//
	// Returns the mutation rate
	//
	float GetMutationRate() { return m_mutationRate; }

	//
	// Returns the crossover rate
	//
	float GetCrossoverRate() { return m_crossoverRate; }

	//
	// Returns the number of generations run so far
	//
	unsigned int GetGeneration() { return m_generation; }

	//
	// Returns the number of chromosome bits in genomes in this pool
	//
	unsigned int GetNumChromosomeBits() { return m_numChromosomeBits; }

	//
	// Returns the current strongest genome
	//
	SPGGenome* GetCurrentStrongestGenome() { return m_currentStrongestGenome; }

	//
	// Returns the chromosome list as a vector of SPGGenomes
	//
	const std::vector<SPGGenome*>& GetGenomeList() { return m_genomeList; }

	//
	// Sets the size of this pool
	//
	void SetPoolSize(unsigned int newSize) { m_poolSize = newSize; }

	//
	// Sets the mutation rate of this pool
	//
	void SetMutationRate(float newRate) { m_mutationRate = newRate; }

	//
	// Sets the crossover rate
	//
	void SetCrossoverRate(float newRate) { m_crossoverRate = newRate; }

	//
	// Sets the generation we are currently on
	//
	void SetGeneration(unsigned int newGen) { m_generation = newGen; }

	//
	// Sets the number of chromosome bits on each genome in this pool
	//
	void SetNumChromosomeBits(unsigned int newBits) { m_numChromosomeBits = newBits; }

	//
	// Set the current strongest genome in this pool
	//
	void SetCurrentStrongestGenome(SPGGenome* newStrongest) { m_currentStrongestGenome = newStrongest; }



	//
	// Goes through and delete all of the genomes in this pool
	//
	void ClearPool();

	//
	// Creates an initial population in this pool
	//
	void CreateInitialPool();

	//
	// 'Breeds' the passed in mother and father, resulting in two new SPGGenomes
	// Returns true if we actually did a crossover, false otherwise
	//
	bool DoCrossover(SPGGenome* mother, SPGGenome* father, SPGGenome*& newChild0, SPGGenome*& newChild1);

	//
	// Chooses a semi-random SPGGenome from this pool, weighted so that the stronger
	// ones will have a higher chance of being picked in a simple roulette wheel selection
	// 
	// If different selection strategies are used further down the line then swap this
	// function out with a strategy pattern
	//
	SPGGenome* ChooseGenome();

	//
	// This function goes through and updates the strength score for each SPGGenome in this pool
	//
	// MUST be implemented by children classes to be used in RunGeneration()
	// 
	virtual void CalcGenomeStrengths() = 0;

	//
	// Runs a generation of this genome pool
	//
	void RunGeneration();

protected:

	// The size of this genome pool
	unsigned int m_poolSize;

	// The mutation rate in this gene pool
	float m_mutationRate;

	// Crossover rate is the probability that when two genomes are being combined
	// there will be an exchange of some part of their chromosomes
	float m_crossoverRate;

	// The generation number that this genome pool is currently on
	unsigned int m_generation;

	// The number of bits in each genome of this pool
	unsigned int m_numChromosomeBits;

	// Store a vector of pointers to genomes
	std::vector<SPGGenome*> m_genomeList;

	// Store a pointer to the current strongest genome
	SPGGenome* m_currentStrongestGenome;
};


#endif